package HTE::Commands;

require 5.005_62;
use strict;
use warnings;

use HTE::CmdTree;

our $VERSION = '0.90';

=head1 NAME

HTE::Commands - Command Syntax for the Host Table Editor program

=head1 SYNOPSIS

  use HTE::Commands;

  my $cmd = HTE::Commands::init;

=head1 DESCRIPTION

This simple module creates a basic command syntax object that ties the
interactive command tree to the actual commands that perform the
tasks.

The main goal of separating the user syntax from the code is to one
day be able to allow local system admins to modify the syntax to
something more friendly, without having to modify the original code.
This may also allow alternate languages to be used.

=head1 Interface Data Structure

The main structure behind the interface creates a hierarchical command
tree.  Each node on the tree will have a description for user help, a
reference to the subroutine that carries out its task or a reference
to the next node in the tree.

=cut

sub new {

  #### There is a more elegant way to build this tree..... XML?

  my $hte = new HTE::CmdTree 'hte';

  my $nh = $hte->node("reload",1);
  $nh->desc("Reload the network tables into memory");
  $nh->cmd("LoadNets");

  my $show = $hte->node("show",1);
  $show->desc("Show information about ip addresses and networks.");

  $nh = $show->node("ip",1);
  $nh->desc('Display host table entry by ip address');
  $nh->cmd('showbyip');

  $nh = $show->node("name",1);
  $nh->desc('Display host table entry by hostname');
  $nh->cmd('showbyhost');

  $nh = $show->node("network",1);
  $nh->desc('Display network entry');
  $nh->cmd('shownet');

  $nh = $show->node("zone",1);
  $nh->desc('Display a zone definition and extra records.');
  $nh->cmd('showzone');

  $nh = $show->node("zones",1);
  $nh->desc('Display a list of all known zones');
  $nh->cmd('ShowZoneList');

  my $build = $hte->node("build",1);
  $build->desc("Build the host table or dns maps");

  my $host = $build->node("host",1);
  $host->desc("Display host tables and DNS maps");

  $nh = $host->node("table",1);
  $nh->desc('Display a standard host table');
  $nh->cmd('table');

  $nh = $host->node("vars",1);
  $nh->desc('Display various report settings');
  $nh->cmd('showvars');

  $nh = $host->node("set",1);
  $nh->desc('Set a variable value:  set varname value');
  $nh->cmd('setvar');

  $nh = $host->node("savevars",1);
  $nh->desc("Save variable settings to the database for $ENV{USER}");
  $nh->cmd('SaveVars');

  $nh = $host->node("resetvars",1);
  $nh->desc('Reset all variables to their default values');
  $nh->cmd('DefaultVars');

  my $dns = $build->node("dns",1);
  $dns->desc("Build the DNS Maps");

  $nh = $dns->node("vars",1);
  $nh->desc('Display various report settings');
  $nh->cmd('showvars');

  $nh = $dns->node("set",1);
  $nh->desc('Set a variable value:  set varname value');
  $nh->cmd('setvar');

  $nh = $dns->node("savevars",1);
  $nh->desc("Save variable settings to the database for $ENV{USER}");
  $nh->cmd('SaveVars');

  $nh = $dns->node("resetvars",1);
  $nh->desc('Reset all variables to their default values');
  $nh->cmd('DefaultVars');

  $nh = $dns->node("maps",1);
  $nh->desc("Build the DNS maps");
  $nh->cmd('named_conf');

  my $create = $hte->node("create",1);
  $create->desc("Create a host record, or network");

  $nh = $create->node("ip",1);
  $nh->desc('Create a new ip address record.  The subnet must exist.');
  $nh->cmd('createip');

  $nh = $create->node("network",1);
  $nh->desc('Create a new network definition.  Use CIDR format');
  $nh->cmd('createnet');

  $nh = $create->node("zone",1);
  $nh->desc('Create a new DNS zone');
  $nh->cmd('createzone');

  $nh = $create->node("zone_record",1);
  $nh->desc('Add a new manual record to a zone. Takes no args.');
  $nh->cmd('createzone_entry');

  my $edit = $hte->node("edit",1);
  $edit->desc("Edit an address entry, network or subnet");

  $nh = $edit->node("ip",1);
  $nh->desc("Edit the specified IP address");
  $nh->cmd('editip');

  $nh = $edit->node("network",1);
  $nh->desc("Edit the specified network address");
  $nh->cmd('editnet');

  $nh = $edit->node("zone",1);
  $nh->desc("Edit the specified zone record");
  $nh->cmd('editzone');

  $nh = $edit->node("zone_entry",1);
  $nh->desc("Edit an extra zone record by id. Show zone gives ids");
  $nh->cmd('editzone_entry');

  my $del = $hte->node("delete",1);
  $del->desc("Delete addresses or networks");

  $nh = $del->node('ip',1);
  $nh->desc("Delete an ip address.");
  $nh->cmd('deladdr');

  $nh = $del->node('network',1);
  $nh->desc("Delete a network and all included ip address.");
  $nh->cmd('delnet');

  $nh = $del->node('zone',1);
  $nh->desc("Delete a zone and all of its extra records");
  $nh->cmd('delzone');

  $nh = $del->node('zone_entry',1);
  $nh->desc("Delete an extra zone entry record");
  $nh->cmd('delzone_entry');

  return $hte;

}

=head1 AUTHOR

Daniel Speers (dan@speers.nu)

=head1 SEE ALSO

HTE(1) - The Host table editor program

=cut

1;
__END__


sub Table {

  my %cmds;

  %cmds = 
    ( show   => {desc => 'Show information about ip addresses and networks.'},
      edit   => {desc => 'Edit the host table or network information.'},
      create => {desc => 'Create a host table entry or network segment.'}
    );

  $cmds{show}->{cmds} = 
    {
     ip      => {desc => 'Display host table entry by ip address',
		 sub  => \&showbyip },
     name    => {desc => 'Display host table entry by hostname',
		 sub  => \&showbyhost },
     devices => {desc => 'Show list of device types',
		 sub  => \&showdevs }
    };


  return \%cmds;
}
